@@ -19,7 +19,7 @@ from Orange.widgets.settings import Setting, ContextSetting, SettingProvider
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.utils.concurrent import TaskState, ConcurrentWidgetMixin
 from Orange.widgets.utils.itemmodels import DomainModel
-from Orange.widgets.utils.colorpalette import ColorPaletteGenerator
+from Orange.widgets.utils.colorpalettes import LimitedDiscretePalette
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.visualize.utils.widget import OWDataProjectionWidget
 from Orange.widgets.visualize.owscatterplotgraph import OWScatterPlotBase
@@ -262,8 +262,8 @@ class OWAnnotateProjectionGraph(OWScatterPlotBase):
         if labels is None:
             return
         for label_per, (x, y), _ in labels:
-            text = "\n".join([l for l, _ in label_per[: self.n_cluster_labels]])
-            ttip = "\n".join([f"{round(p * 100)}%  {l}" for l, p in label_per])
+            text = "\n".join([label for label, _ in label_per[: self.n_cluster_labels]])
+            ttip = "\n".join([f"{round(p * 100)}%  {label}" for label, p in label_per])
             item = CenteredTextItem(self.view_box, x, y, text, ttip)
             self.plot_widget.addItem(item)
             self.cluster_labels_items.append(item)
@@ -321,8 +321,6 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
 
     GRAPH_CLASS = OWAnnotateProjectionGraph
     graph = SettingProvider(OWAnnotateProjectionGraph)
-    # remove this line when https://github.com/biolab/orange3/pull/3863 is released
-    left_side_scrolling = True
 
     attr_x = ContextSetting(None)
     attr_y = ContextSetting(None)
@@ -363,7 +361,7 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
         missing_cell_type = Msg("'Cell Type' is missing in Genes table.")
         missing_tax_id_genes = Msg(f"'{TAX_ID}' is missing in Genes table. " f"Try using 'Marker Genes' widget.")
         different_tax_id = Msg(f"Data and Genes appear to describe different " f"organisms (mismatching {TAX_ID}).")
-        same_axis_features = Msg(f"Selected features for Axis x and " f"Axis y should differ.")
+        same_axis_features = Msg("Selected features for Axis x and " "Axis y should differ.")
 
     class Error(OWDataProjectionWidget.Error):
         no_reference_data = Msg("Missing Reference data on input.")
@@ -423,7 +421,7 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
             'sendSelectedValue': True,
             'contentsLength': 14,
         }
-        box = gui.vBox(self.controlArea, True)
+        box = gui.vBox(self.controlArea, box='Axes')
         ord = (DomainModel.METAS, DomainModel.ATTRIBUTES, DomainModel.CLASSES)
         mod = DomainModel(ord, valid_types=ContinuousVariable)
         gui.comboBox(
@@ -432,6 +430,8 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
         gui.comboBox(
             box, self, "attr_y", label="Axis y:", model=mod, callback=self.__axis_attr_changed, **common_options
         )
+
+        box = gui.vBox(self.controlArea, box='Annotation')
         gui.comboBox(
             box,
             self,
@@ -463,13 +463,16 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
 
     @property
     def effective_variables(self):
-        return self.reference_data.domain.attributes
+        attr_x, attr_y = self.attr_x, self.attr_y
+        if attr_x and attr_y:
+            if attr_x == attr_y:
+                return [attr_x]
+            return [attr_x, attr_y]
+        return []
 
     @property
     def effective_data(self):
-        return self.reference_data.transform(
-            Domain(self.effective_variables, self.reference_data.domain.class_vars, self.reference_data.domain.metas)
-        )
+        return self.reference_data.transform(Domain(self.effective_variables))
 
     @property
     def can_annotate(self):
@@ -540,7 +543,7 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
         if self.task is not None:
             self.cancel()
             self.run_button.setText("Resume")
-            self.commit()
+            self.commit.deferred()
         else:
             self._run()
 
@@ -586,7 +589,7 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
         if result.clusters.epsilon is not None:
             self.epsilon = result.clusters.epsilon
         self.run_button.setText("Start")
-        self.commit()
+        self.commit.deferred()
 
     def on_exception(self, ex: Exception):
         self.Error.proj_error(ex)
@@ -786,7 +789,7 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
         return (
             self.clusters.table.domain["Clusters"].palette
             if hasattr(self.clusters.table.domain["Clusters"], "palette")
-            else ColorPaletteGenerator(number_of_colors=len(colors), rgb_colors=colors)
+            else LimitedDiscretePalette(number_of_colors=len(colors))
         )
 
     def get_cluster_hulls(self):
@@ -831,9 +834,10 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
 
         def get_augmented_data(table, scores_x, clusters_x):
             new_table = table.transform(domain)
-            if scores_x is not None:
-                new_table.metas[:, :n_sco] = scores_x
-            new_table.metas[:, n_sco:n_clu] = clusters_x
+            with new_table.unlocked(new_table.metas):
+                if scores_x is not None:
+                    new_table.metas[:, :n_sco] = scores_x
+                new_table.metas[:, n_sco:n_clu] = clusters_x
             return new_table
 
         n_sco = self.scores.table.X.shape[1]
@@ -872,8 +876,9 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
         metas = chain(domain.metas, [source_attr])
         domain = Domain(domain.attributes, domain.class_vars, metas)
         data = data.transform(domain)
-        data.metas[: len(self.reference_data), -1] = 0
-        data.metas[len(self.reference_data) :, -1] = 1
+        with data.unlocked(data.metas):
+            data.metas[: len(self.reference_data), -1] = 0
+            data.metas[len(self.reference_data) :, -1] = 1
         return data
 
     def _get_send_report_caption(self):
@@ -900,6 +905,7 @@ class OWAnnotateProjection(OWDataProjectionWidget, ConcurrentWidgetMixin):
 
 if __name__ == "__main__":
     from Orange.projection import PCA
+
     from orangecontrib.bioinformatics.utils import serverfiles
 
     data_path = "https://datasets.biolab.si/sc/aml-1k.tab.gz"
