@@ -2,29 +2,25 @@
 import sys
 from types import SimpleNamespace
 from typing import Any, Optional, DefaultDict
-from functools import lru_cache
-from collections import OrderedDict, defaultdict
+from collections import defaultdict
 
-import numpy as np
 import requests
 
-from AnyQt.QtGui import QFont, QColor
-from AnyQt.QtCore import Qt, QSize, QVariant, QModelIndex
+from AnyQt.QtCore import Qt
 from AnyQt.QtWidgets import (
-    QStyle,
     QSplitter,
-    QTableView,
     QTreeWidget,
     QTreeWidgetItem,
     QAbstractItemView,
     QAbstractScrollArea,
 )
 
+from orangewidget.utils.itemdelegates import DataDelegate
+
 from Orange.data import Table
 from Orange.widgets.gui import (
     LinkRole,
     IndicatorItemDelegate,
-    LinkStyledItemDelegate,
     rubber,
     lineEdit,
     separator,
@@ -33,21 +29,32 @@ from Orange.widgets.gui import (
     widgetLabel,
     radioButtonsInBox,
 )
-from Orange.widgets.utils import itemmodels
 from Orange.widgets.widget import Msg, OWWidget
 from Orange.widgets.settings import Setting
 from Orange.widgets.utils.signals import Output
+from Orange.widgets.utils.tableview import TableView
 from Orange.widgets.utils.concurrent import TaskState, ConcurrentWidgetMixin
 
 from orangecontrib.bioinformatics.geo import is_cached, pubmed_url, local_files
-from orangecontrib.bioinformatics.geo.dataset import GDSInfo, get_samples, dataset_download
+from orangecontrib.bioinformatics.geo.dataset import (
+    GDSInfo,
+    get_samples,
+    dataset_download,
+)
+from orangecontrib.bioinformatics.widgets.utils.gui import FilterProxyModel
+from orangecontrib.bioinformatics.widgets.utils.itemmodels import TableModel
+from orangecontrib.bioinformatics.widgets.utils.itemdelegates import (
+    LinkStyledItemDelegate,
+)
 
 
 class Result(SimpleNamespace):
     gds_dataset: Optional[Table] = None
 
 
-def run_download_task(gds_id: str, samples: DefaultDict[str, list], transpose: bool, state: TaskState):
+def run_download_task(
+    gds_id: str, samples: DefaultDict[str, list], transpose: bool, state: TaskState
+):
     res = Result()
     current_iter = 0
     max_iter = 102
@@ -58,162 +65,139 @@ def run_download_task(gds_id: str, samples: DefaultDict[str, list], transpose: b
         state.set_progress_value(100 * (current_iter / max_iter))
 
     state.set_status("Downloading...")
-    res.gds_dataset = dataset_download(gds_id, samples, transpose=transpose, callback=callback)
+    res.gds_dataset = dataset_download(
+        gds_id, samples, transpose=transpose, callback=callback
+    )
     return res
 
 
-class GEODatasetsModel(itemmodels.PyTableModel):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+class GEODatasetsModel(TableModel):
+    (
+        indicator_col,
+        gds_id_col,
+        pubmed_id_col,
+        organism_col,
+        samples_col,
+        features_col,
+        genes_col,
+        subsets_col,
+        title_col,
+    ) = range(9)
 
-        self.setHorizontalHeaderLabels(
-            ['', 'ID', 'PubMedID', 'Organism', 'Samples', 'Features', 'Genes', 'Subsets', 'Title']
-        )
+    def __init__(self, gds: GDSInfo):
+        items = list(gds.values())
+        pubmed_ids = [g.get("pubmed_id", None) for g in items]
 
-        (
-            self.indicator_col,
-            self.gds_id_col,
-            self.pubmed_id_col,
-            self.organism_col,
-            self.samples_col,
-            self.features_col,
-            self.genes_col,
-            self.subsets_col,
-            self.title_col,
-        ) = range(9)
+        def title(name: str):
+            return {Qt.DisplayRole: name}
 
-        self.info = None
-        self.table = None
-        self._sort_column = self.gds_id_col
-        self._sort_order = Qt.AscendingOrder
+        def pubmed_id(row: int) -> Optional[str]:
+            pubmed = pubmed_ids[row]
+            if isinstance(pubmed, list):
+                return pubmed[0] if len(pubmed) > 0 else None
+            else:
+                return pubmed
 
-        self.font = QFont()
-        self.font.setUnderline(True)
-        self.color = QColor(Qt.blue)
+        def render_pubmed_url(row):
+            id_ = pubmed_id(row)
+            if id_:
+                return pubmed_url.format(id_)
+            return None
 
-        @lru_cache(maxsize=10000)
-        def _row_instance(row, column):
-            return self[int(row)][int(column)]
-
-        self._row_instance = _row_instance
-
-    def initialize(self, info: OrderedDict):
-        self.info = info
-
-        def _gds_to_row(gds: dict):
-            gds_id = gds['name']
-            title = gds['title']
-            organism = gds['sample_organism']
-            samples = len(get_samples(gds))
-            features = gds['variables']
-            genes = gds['genes']
-            subsets = len(gds['subsets'])
-
-            pubmed = gds.get('pubmed_id', '')
-            pubmed_id = pubmed
-            if isinstance(pubmed, list) and len(pubmed) > 0:
-                pubmed_id = pubmed[0]
-
-            return [
-                ' ' if is_cached(gds_id) else '',
-                gds_id,
-                pubmed_id,
-                organism,
-                samples,
-                features,
-                genes,
-                subsets,
-                title,
-            ]
-
-        self.table = np.asarray([_gds_to_row(gds) for gds in info.values()])
-        self.show_table()
-
-    def _argsortData(self, data: np.ndarray, order) -> Optional[np.ndarray]:
-        if not data.size:
-            return
-
-        # store order choice.
-        self._sort_column = column = self.sortColumn()
-        self._sort_order = self.sortOrder()
+        columns = [
+            TableModel.Column(
+                title(""),
+                {
+                    Qt.DisplayRole: lambda row: " "
+                    if is_cached(items[row]["name"])
+                    else "",
+                    Qt.UserRole: lambda row: items[row],
+                },
+            ),
+            TableModel.Column(
+                title("ID"),
+                {
+                    Qt.DisplayRole: lambda row: items[row]["name"],
+                    Qt.UserRole: lambda row: items[row],
+                },
+            ),
+            TableModel.Column(
+                title("PubMedID"),
+                {
+                    Qt.DisplayRole: pubmed_id,
+                    LinkRole: render_pubmed_url,
+                    Qt.ToolTipRole: render_pubmed_url,
+                },
+            ),
+            TableModel.Column(
+                title("Organism"),
+                {
+                    Qt.DisplayRole: lambda row: items[row]["sample_organism"],
+                },
+            ),
+            TableModel.Column(
+                title("Samples"),
+                {
+                    Qt.DisplayRole: lambda row: len(get_samples(items[row])),
+                },
+            ),
+            TableModel.Column(
+                title("Features"),
+                {
+                    Qt.DisplayRole: lambda row: items[row]["variables"],
+                },
+            ),
+            TableModel.Column(
+                title("Genes"),
+                {
+                    Qt.DisplayRole: lambda row: items[row]["genes"],
+                },
+            ),
+            TableModel.Column(
+                title("Subsets"),
+                {
+                    Qt.DisplayRole: lambda row: len(items[row]["subsets"]),
+                },
+            ),
+            TableModel.Column(
+                title("Title"),
+                {
+                    Qt.DisplayRole: lambda row: items[row]["title"],
+                    Qt.ToolTipRole: lambda row: items[row]["title"],
+                },
+            ),
+        ]
+        super().__init__(len(items), columns)
+        self.info = gds
 
+    def sortColumnData(self, column):
         if column == self.gds_id_col:
-            data = np.char.replace(data, 'GDS', '')
-            data = data.astype(int)
-
-        elif column in (self.samples_col, self.features_col, self.genes_col, self.subsets_col, self.pubmed_id_col):
-            data[data == ''] = '0'
-            data = data.astype(int)
-
-        indices = np.argsort(data, kind='mergesort')
-        if order == Qt.DescendingOrder:
-            indices = indices[::-1]
-        return indices
-
-    def columnCount(self, parent=QModelIndex()):
-        return 0 if parent.isValid() else self._table.shape[1]
-
-    def data(
-        self,
-        index,
-        role,
-        _str=str,
-        _Qt_DisplayRole=Qt.DisplayRole,  # noqa: N803
-        _Qt_EditRole=Qt.EditRole,
-        _Qt_FontRole=Qt.FontRole,
-        _Qt_ForegroundRole=Qt.ForegroundRole,
-        _LinkRolee=LinkRole,
-        _recognizedRoles=frozenset([Qt.DisplayRole, Qt.EditRole, Qt.FontRole, Qt.ForegroundRole, LinkRole]),
-    ):
-
-        if role not in _recognizedRoles:
-            return None
-
-        row, col = index.row(), index.column()
-        if not 0 <= row <= self.rowCount():
-            return None
-        row = self.mapToSourceRows(row)
-
-        try:
-            # value = self[row][col]
-            value = self._row_instance(row, col)
-        except IndexError:
-            return
-
-        if role == Qt.DisplayRole:
-            return QVariant(str(value))
-        elif role == Qt.ToolTipRole:
-            return QVariant(str(value))
-
-        if col == self.pubmed_id_col:
-            if role == _Qt_ForegroundRole:
-                return self.color
-            elif role == _Qt_FontRole:
-                return self.font
-            elif role == _LinkRolee:
-                return pubmed_url.format(value)
-
-    def get_row_index(self, gds_name):
-        # test = self._table[self._table[:, 1] == gds_name, :]
-        rows, _ = np.where(np.isin(self._table, gds_name))
-        if rows is not None and len(rows) > 0:
-            return self.mapFromSourceRows(rows[0])
-
-    def filter_table(self, filter_pattern: str):
-        selection = np.full(self.table.shape, True)
-        for search_word in filter_pattern.split():
-            match_result = np.core.defchararray.find(np.char.lower(self.table), search_word.lower()) >= 0
-            selection = selection & match_result
-        return selection
+            items = [g["name"] for g in self.info.values()]
+            items = [int(name.strip('GDS')) for name in items]
+            return items
+        if column == self.pubmed_id_col:
+            items = [
+                self.data(self.index(i, column), Qt.DisplayRole)
+                for i in self.mapFromSourceRows(range(self.rowCount()))
+            ]
+            items = [int(pid or 0) for pid in items]
+            return items
+        return super().sortColumnData(column)
 
     def update_cache_indicator(self):
-        self.table[:, 0] = [' ' if is_cached(gid) else '' for gid in self.table[:, self.gds_id_col]]
+        self.dataChanged.emit(self.index(0, 0), self.index(self.rowCount() - 1, 0))
 
-    def show_table(self, filter_pattern=''):
-        # clear cache if model changes
-        self._row_instance.cache_clear()
-        self.wrap(self.table[self.filter_table(filter_pattern).any(axis=1), :])
-        self.sort(self._sort_column, self._sort_order)
+
+class FilterProxyModel(FilterProxyModel):
+    def sort(self, column: int, order: Qt.SortOrder = Qt.AscendingOrder) -> None:
+        """
+        Reimplemented.
+
+        Dispatch the sorting to the source model.
+        """
+        source = self.sourceModel()
+        if source is not None:
+            source.sort(column, order)
 
 
 class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
@@ -257,13 +241,20 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
             return
 
         self.gds_data: Optional[Table] = None
+        self.__updating_filter = False
 
         # Control area
         box = widgetBox(self.controlArea, 'Info', addSpace=True)
         self.infoBox = widgetLabel(box, 'Initializing\n\n')
 
         box = widgetBox(self.controlArea, 'Output', addSpace=True)
-        radioButtonsInBox(box, self, 'genes_as_rows', ['Samples in rows', 'Genes in rows'], callback=self._run)
+        radioButtonsInBox(
+            box,
+            self,
+            'genes_as_rows',
+            ['Samples in rows', 'Genes in rows'],
+            callback=self._run,
+        )
         separator(box)
 
         rubber(self.controlArea)
@@ -273,7 +264,12 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
 
         # Filter widget
         self.filter = lineEdit(
-            self.mainArea, self, 'search_pattern', 'Filter:', callbackOnType=True, callback=self._apply_filter
+            self.mainArea,
+            self,
+            'search_pattern',
+            'Filter:',
+            callbackOnType=True,
+            callback=self._apply_filter,
         )
         self.mainArea.layout().addWidget(self.filter)
 
@@ -282,38 +278,38 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
         self.mainArea.layout().addWidget(splitter_vertical)
 
         # set table view
-        self.table_view = QTableView(splitter_vertical)
-        self.table_view.setShowGrid(False)
-        self.table_view.setSortingEnabled(True)
+        self.table_view = TableView(
+            splitter_vertical,
+            showGrid=False,
+            sortingEnabled=True,
+            alternatingRowColors=True,
+            selectionBehavior=QAbstractItemView.SelectRows,
+            selectionMode=QAbstractItemView.SingleSelection,
+            sizeAdjustPolicy=QAbstractScrollArea.AdjustToContents,
+        )
         self.table_view.sortByColumn(1, Qt.AscendingOrder)
-        self.table_view.setAlternatingRowColors(True)
         self.table_view.verticalHeader().setVisible(False)
-        self.table_view.setSelectionBehavior(QAbstractItemView.SelectRows)
-        self.table_view.setSelectionMode(QAbstractItemView.SingleSelection)
         self.table_view.viewport().setMouseTracking(True)
-        self.table_view.setSizeAdjustPolicy(QAbstractScrollArea.AdjustToContents)
 
-        self.table_model = GEODatasetsModel()
-        self.table_model.initialize(self.gds_info)
-        self.table_view.setModel(self.table_model)
+        self.table_model = GEODatasetsModel(self.gds_info)
+        self.proxy_model = FilterProxyModel()
+        self.proxy_model.setSourceModel(self.table_model)
+        self.table_view.setModel(self.proxy_model)
 
         self.table_view.horizontalHeader().setStretchLastSection(True)
         self.table_view.resizeColumnsToContents()
 
-        v_header = self.table_view.verticalHeader()
-        option = self.table_view.viewOptions()
-        size = self.table_view.style().sizeFromContents(QStyle.CT_ItemViewItem, option, QSize(20, 20), self.table_view)
-
-        v_header.setDefaultSectionSize(size.height() + 2)
-        v_header.setMinimumSectionSize(5)
-
         # set item delegates
+        self.table_view.setItemDelegate(DataDelegate(self.table_view))
         self.table_view.setItemDelegateForColumn(
             self.table_model.pubmed_id_col, LinkStyledItemDelegate(self.table_view)
         )
-        self.table_view.setItemDelegateForColumn(self.table_model.gds_id_col, LinkStyledItemDelegate(self.table_view))
         self.table_view.setItemDelegateForColumn(
-            self.table_model.indicator_col, IndicatorItemDelegate(self.table_view, role=Qt.DisplayRole)
+            self.table_model.gds_id_col, LinkStyledItemDelegate(self.table_view)
+        )
+        self.table_view.setItemDelegateForColumn(
+            self.table_model.indicator_col,
+            IndicatorItemDelegate(self.table_view, role=Qt.DisplayRole),
         )
 
         splitter_horizontal = QSplitter(Qt.Horizontal, splitter_vertical)
@@ -327,18 +323,24 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
         # Sample Annotations Widget
         box = widgetBox(splitter_horizontal, 'Sample Annotations')
         self.annotations_widget = QTreeWidget(box)
-        self.annotations_widget.setHeaderLabels(['Type (Sample annotations)', 'Sample count'])
+        self.annotations_widget.setHeaderLabels(
+            ['Type (Sample annotations)', 'Sample count']
+        )
         self.annotations_widget.setRootIsDecorated(True)
         box.layout().addWidget(self.annotations_widget)
         self._annotations_updating = False
-        self.annotations_widget.itemChanged.connect(self.on_annotation_selection_changed)
+        self.annotations_widget.itemChanged.connect(
+            self.on_annotation_selection_changed
+        )
         self.splitters = splitter_vertical, splitter_horizontal
 
         for sp, setting in zip(self.splitters, self.splitter_settings):
             sp.splitterMoved.connect(self._splitter_moved)
             sp.restoreState(setting)
 
-        self.table_view.selectionModel().selectionChanged.connect(self.on_gds_selection_changed)
+        self.table_view.selectionModel().selectionChanged.connect(
+            self.__on_selection_changed
+        )
         self._apply_filter()
 
         self.commit()
@@ -347,7 +349,9 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
         self.splitter_settings = [bytes(sp.saveState()) for sp in self.splitters]
 
     def _set_description_widget(self):
-        self.description_widget.setText(self.selected_gds.get('description', 'Description not available.'))
+        self.description_widget.setText(
+            self.selected_gds.get('description', 'Description not available.')
+        )
 
     def _set_annotations_widget(self, gds):
         self._annotations_updating = True
@@ -378,16 +382,28 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
 
     def _set_selection(self):
         if self.selected_gds is not None:
-            index = self.table_model.get_row_index(self.selected_gds.get('name'))
+            index = None
+            ids = self.proxy_model.match(
+                self.proxy_model.index(0, 1),
+                Qt.DisplayRole,
+                self.selected_gds["name"],
+                1,
+                Qt.MatchCaseSensitive | Qt.MatchExactly,
+            )
+            if ids:
+                index = ids[0]
+
             if index is not None:
                 self.table_view.selectionModel().blockSignals(True)
-                self.table_view.selectRow(index)
+                self.table_view.selectRow(index.row())
                 self._handle_selection_changed()
                 self.table_view.selectionModel().blockSignals(False)
 
     def _handle_selection_changed(self):
-        if self.table_model.table is not None:
-            selection = self.table_view.selectionModel().selectedRows(self.table_model.gds_id_col)
+        if self.table_model is not None:
+            selection = self.table_view.selectionModel().selectedRows(
+                self.table_model.gds_id_col
+            )
             selected_gds_name = selection[0].data() if len(selection) > 0 else None
 
             if selected_gds_name:
@@ -401,19 +417,54 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
             self.update_info()
 
     def _apply_filter(self):
-        if self.table_model.table is not None:
-            self.table_model.show_table(filter_pattern=str(self.search_pattern))
-            self._set_selection()
-            self.update_info()
+        keys = self.search_pattern.lower().split()
+
+        def filter(gds):
+            source = (gds["name"], gds["title"], gds["sample_organism"])
+            source = [t.lower() for t in source]
+            return all(any(k in s for s in source) for k in keys)
+
+        changed = False
+
+        def current_changed():
+            nonlocal changed
+            changed = True
+
+        selmodel = self.table_view.selectionModel()
+        selmodel.selectionChanged.connect(current_changed)
+        try:
+            self.__updating_filter = True
+            self.proxy_model.set_filters(
+                [
+                    FilterProxyModel.Filter(
+                        self.table_model.gds_id_col, Qt.UserRole, filter
+                    )
+                ]
+            )
+            if changed:
+                self.table_view.clearSelection()
+                self._handle_selection_changed()
+        finally:
+            selmodel.selectionChanged.disconnect(current_changed)
+            self.__updating_filter = False
+
+        self._set_selection()
 
     def _run(self):
         self.Warning.using_local_files.clear()
         if self.selected_gds is not None:
             self.gds_data = None
             self.start(
-                run_download_task, self.selected_gds.get('name'), self.get_selected_samples(), self.genes_as_rows
+                run_download_task,
+                self.selected_gds.get('name'),
+                self.get_selected_samples(),
+                self.genes_as_rows,
             )
 
+    def __on_selection_changed(self):
+        if not self.__updating_filter:
+            self.on_gds_selection_changed()
+
     def on_gds_selection_changed(self):
         self._handle_selection_changed()
         self.commit()
@@ -434,7 +485,9 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
 
     def update_info(self):
         all_gds = len(self.table_model.info)
-        text = "{} datasets\n{} datasets cached\n".format(all_gds, len(local_files.listfiles()))
+        text = "{} datasets\n{} datasets cached\n".format(
+            all_gds, len(local_files.listfiles())
+        )
         filtered = self.table_view.model().rowCount()
         if all_gds != filtered:
             text += "{} after filtering".format(filtered)
@@ -455,8 +508,7 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
         """
 
         def childiter(item):
-            """ Iterate over the children of an QTreeWidgetItem instance.
-            """
+            """Iterate over the children of an QTreeWidgetItem instance."""
             for i in range(item.childCount()):
                 yield item.child(i)
 
@@ -497,7 +549,6 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
 
         if self.gds_info:
             self.table_model.update_cache_indicator()
-            self._apply_filter()
 
         self.Outputs.gds_data.send(self.gds_data)
 
@@ -528,12 +579,16 @@ class OWGEODatasets(OWWidget, ConcurrentWidgetMixin):
         self.report_name("Sample annotations")
         subsets = defaultdict(list)
         for subset in self.selected_gds['subsets']:
-            subsets[subset['type']].append((subset['description'], len(subset['sample_id'])))
+            subsets[subset['type']].append(
+                (subset['description'], len(subset['sample_id']))
+            )
         self.report_html += "<ul>"
         for _type in subsets:
             self.report_html += "<b>" + _type + ":</b></br>"
             for desc, count in subsets[_type]:
-                self.report_html += 9 * "&nbsp" + "<b>{}:</b> {}</br>".format(desc, count)
+                self.report_html += 9 * "&nbsp" + "<b>{}:</b> {}</br>".format(
+                    desc, count
+                )
         self.report_html += "</ul>"
 
 
